import execution_state
import os
import math
from zplot import *

class CloseListener():
  def __init__(self):
    pass

  def GetName(self):
    raise Exception('Must override')

  def Close(fd):
    raise Exception('Must override')

  def OutputToPath(self, path):
    raise Exception('Must override')
    

class FileSizeCDF(CloseListener):
  def __init__(self):
    self.total_file_count = 0
    self.total_file_bytes = 0
    self.size_counts = {}

  def GetName(self):
    return 'file_size_cdf'

  def Close(self, fd):
    if fd.TotalIO() == 0:
      return
    self.total_file_count += 1
    self.total_file_bytes += fd.inode.size
    if not fd.inode.size in self.size_counts:
      self.size_counts[fd.inode.size] = 1
    else:
      self.size_counts[fd.inode.size] += 1

  def OutputToDir(self, out_dir, experiment):
    f = open(os.path.join(out_dir, '%s.txt' % experiment), 'w')
    curr_count_sum = 0
    curr_byte_sum = 0
    sizes = sorted(self.size_counts.keys())
    for size in sizes:
      curr_count_sum += self.size_counts[size]
      curr_byte_sum += self.size_counts[size] * size
      log_size = math.log(size, 10)
      f.write('%d %f %f\n' %
              (log_size,
               float(curr_count_sum) / self.total_file_count,
              float(curr_byte_sum) / self.total_file_bytes))
    # max value
    f.write('%d %f %f\n' %
            (math.log(2**30, 10),
             float(curr_count_sum) / self.total_file_count,
             float(curr_byte_sum) / self.total_file_bytes))
    f.close()
    self.GenGraph(out_dir, experiment)

  def GenGraph(self, out_dir, experiment):
    eps_file = os.path.join(out_dir, '%s.eps' % experiment)
    c = postscript(title=eps_file, dimensions=[380,290])
    d = drawable(canvas=c, xrange=[0,9], yrange=[0,1], dimensions=[330,200], coord=[38,40])
    t = table(file=os.path.join(out_dir, '%s.txt' % experiment))
    p = plotter()
    L = legend()

    xlabels = [['0B', 0],
               ['32B', math.log(2**5,10)],
               ['1KB', math.log(2**10,10)],
               ['32KB', math.log(2**15,10)],
               ['1MB', math.log(2**20,10)],
               ['32MB', math.log(2**25,10)],
               ['1GB', math.log(2**30,10)]]
    axis(drawable=d, style='x',
         xmanual=xlabels, xtitle='File Size')
    ylabels = [['0%', 0],
               ['20%', 0.2],
               ['40%', 0.4],
               ['60%', 0.6],
               ['80%', 0.8],
               ['100%', 1.0]]
    axis(drawable=d, style='y',
         ymanual=ylabels)

    p.line(drawable     = d,
           table        = t, 
           xfield       = 'c0', 
           yfield       = 'c1', 
           stairstep    = True,
           legend       = L,
           legendtext   = 'Count',
           linewidth    = 1.5,
           linecolor    = '0.6,0.6,0.6'
           )
    p.line(drawable     = d,
           table        = t, 
           xfield       = 'c0', 
           yfield       = 'c2', 
           stairstep    = True,
           legend       = L,
           legendtext   = 'Bytes',
           linewidth    = 1.5,
           linecolor    = 'black',
           linedash     = [3,3]
           )

    L.draw(drawable=d, coord=[7.8,0.12])
    c.render()

    



